package storybook.toolkit.html;

import i18n.I18N;
import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.text.BadLocationException;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLWriter;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import storybook.App;
import storybook.toolkit.TextUtil;
import storybook.toolkit.swing.ColorUtil;

public class Html {

	private static final String TT = "Html";

	public static String emptyLines(int n) {
		StringBuilder p = new StringBuilder();
		for (int i = 0; i < n; i++) {
			p.append(P_EMPTY);
		}
		return p.toString();
	}

	public static boolean isEmpty(String notes) {
		return htmlToText(notes).isEmpty();
	}

	/**
	 * get list of Body fragments max size=25000
	 *
	 * @param text
	 * @param t
	 * @return
	 */
	public static List<String> getFragments(String text) {
		String ttext = text;
		int max = 25000;
		List<String> list = new ArrayList<>();
		String[] frags = text.split("<p");
		List<String> l2 = new ArrayList<>();
		if (frags.length > 1) {
			for (String s : frags) {
				if (l2.toString().length() + ("<p" + s).length() > max) {
					list.add(l2.toString());
					l2 = new ArrayList<>();
					l2.add("<p" + s);
				}
			}
			list.add(l2.toString());
		} else {
			list.add(text);
		}
		return list;
	}

	public static String removeTag(String html, String tag) {
		Document doc = Jsoup.parse(html);
		for (Element element : doc.select(tag)) {
			element.remove();
		}
		String t = doc.body().outerHtml()
				.replace("<body>", "")
				.replace("</body>", "");
		if (t.startsWith("\n")) {
			t = t.substring(1);
		}
		if (t.endsWith("\n")) {
			t = t.substring(0, t.length() - 1);
		}
		return t.trim();
	}

	private Html() {
		throw new IllegalStateException("Utility class Html");
	}

	public static final String TYPE = "text/html",
			DOCTYPE = "<!DOCTYPE html>\n",
			ALIGN_CENTER = "text-align: center;",
			ALIGN_LEFT = "text-align: left;",
			ALIGN_RIGHT = "text-align: right;",
			B = "<b>", B_END = "</b>",
			BEGIN = "<html>\n", BEGIN_LANG = "<html lang=\"" + Locale.getDefault().getLanguage() + "\">\n",
			BODY = "<body>\n", BODY_END = "</body>\n",
			BR = "<br>\n",
			CSS_BEGIN = "<style>\n", CSS_END = "</style>\n",
			DIV = "<div>", DIV_END = "</div>",
			END = "</html>\n",
			EXT = ".html",
			FONT_FAMILY = "font-family",
			FONT_SIZE = "font-size",
			FONT_SIZE_DEFAULT = "html {fontsize: " + App.getInstance().fontGetDefault().getSize() + ";}",
			FONT_STYLE = "font-style",
			HEAD = "<head>\n", HEAD_END = "</head>\n",
			META_CONTENT = "<meta name=\"keywords\" content=\"HTML\">\n",
			META_UTF8 = "<meta charset=\"utf-8\">\n",
			HR = "<hr>\n",
			I = "<i>", I_END = "</i>",
			LI = "<li>", LI_END = "</li>\n",
			MARGIN_BOTTOM = "margin-bottom",
			MARGIN_LEFT = "margin-left",
			MARGIN_RIGHT = "margin-right",
			MARGIN_TOP = "margin-top",
			PADDING_LEFT = "padding-left",
			PADDING_RIGHT = "padding-right",
			P_CENTER = "<p style=\"" + ALIGN_CENTER + "\">",
			P_EMPTY = "<p></p>\n",
			P_END = "</p>\n",
			P = "<p>",
			STYLE = "<style type=\"text/css\">\n",
			STYLE_END = "</style>\n",
			TABLE = "<table>\n", TABLE_END = "</table>\n",
			TABLE_STYLE = "<style>table,td,th {border: 1px solid black;border-collapse: separate;border-spacing: 0px;}</style>\n",
			TD = "<td>", TD_END = "</td>\n",
			TEXT_ALIGN = "text-align",
			TR = "<tr>\n", TR_END = "</tr>\n",
			UL = "<ul>\n", UL_END = "</ul>\n",
			U = "<u>", U_END = "</u>";

	/**
	 * return a cleaning HTML string
	 *
	 * @param html
	 * @return
	 */
	public static String getCleanHtml(String html) {
		StringBuilder buf = new StringBuilder();
		appendCleanHtml(buf, html);
		return buf.toString();
	}

	/**
	 * return a cleaning HTML string
	 *
	 * @param buf
	 * @param html
	 */
	public static void appendCleanHtml(StringBuilder buf, String html) {
		// remove new lines
		html = html.replace("\n", "");
		// replace empty div tags with paragraphs: "<div>\s*</div>"
		html = html.replaceAll(DIV + "\\s*" + DIV_END, P_EMPTY);
		// body>(.*)</body
		Pattern p = Pattern.compile("body>(.*)</body");
		Matcher m = p.matcher(html);
		if (m.find()) {
			html = m.group(1);
		}
		html = html.trim();
		// not in any tag at all, so build one
		boolean addPara = false;
		if (!html.startsWith("<")) {
			addPara = true;
			buf.append(P);
		}
		buf.append(html);
		if (addPara) {
			buf.append(P_END);
		}
	}

	/**
	 * return HTML content from a HTMLDocument
	 *
	 * @param doc
	 * @return
	 */
	public static String getContent(HTMLDocument doc) {
		try {
			StringWriter writer = new StringWriter();
			HTMLWriter htmlWriter = new HtmlBodyWriter(writer, doc);
			htmlWriter.write();
			return writer.toString();
		} catch (IOException | BadLocationException e) {
		}
		return "";
	}

	/**
	 * change links to relative one
	 *
	 * @param html
	 * @param dir
	 * @return
	 */
	public static String setToRelative(String html, String dir) {
		if (dir == null || dir.isEmpty()) {
			return (html);
		}
		String r = html;
		if (html.contains("file:")) {
			r = html.replace("file://", "");
			r = r.replace(dir, "");
		}
		return (r);
	}

	/**
	 * return href of a link
	 *
	 * @param html
	 * @return
	 */
	public static String findHref(String html) {
		Pattern pattern = Pattern
				.compile("(((file|http|https)://)|(www\\.))+(([a-zA-Z0-9\\._-]+\\.[a-zA-Z]{2,6})|([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}))(/[a-zA-Z0-9\\&amp;%_\\./-~-]*)?");
		Matcher matcher = pattern.matcher(html);
		if (matcher.find()) {
			return matcher.group();
		}
		return null;
	}

	public static String getAttribute(String key, String value) {
		return " " + key + "=\"" + value + "\"";
	}

	/**
	 * return a BODY with some attribute
	 *
	 * @param complement
	 * @return
	 */
	public static String getBody(String complement) {
		return "<body " + complement + ">";
	}

	/**
	 * return STYLE definition
	 *
	 * @param style
	 * @return
	 */
	public static String getStyle(String style) {
		return " style=\"" + style + "\"";
	}

	public static String intoHTML(String str) {
		return BEGIN + str + END;
	}

	public static String indent(int n) {
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < n; i++) {
			b.append("\t");
		}
		return b.toString();
	}

	/**
	 * return title/subtitle (H)
	 *
	 * @param level
	 * @param text
	 * @param style
	 * @return
	 */
	public static String intoH(Integer level, String text, String... style) {
		String h = "h" + level.toString();
		StringBuilder b = new StringBuilder("<").append(h);
		if (style != null && style.length > 0) {
			b.append(getStyle(style[0]));
		}
		b.append(">")
				.append(text)
				.append("</").append(h).append(">\n");
		return b.toString();
	}

	/**
	 * return a link (A) with optional name, link and text
	 *
	 * @param name: maybe empty
	 * @param href: maybe empty
	 * @param text
	 * @return
	 */
	public static String intoA(String name, String href, String text) {
		return intoA(name, href, text, "");
	}

	/**
	 * return a link (A) with optional name, link and text
	 *
	 * @param name: maybe empty
	 * @param href: maybe empty
	 * @param text
	 * @param cpl : complement like "class=...", or "style=..."
	 * @return
	 */
	public static String intoA(String name, String href, String text, String cpl) {
		StringBuilder b = new StringBuilder("<a");
		if (!name.isEmpty()) {
			b.append(getAttribute("name", name));
		}
		if (!href.isEmpty()) {
			b.append(getAttribute("href", href));
		}
		if (!cpl.isEmpty()) {
			b.append(cpl);
		}
		if (!text.isEmpty()) {
			b.append(">").append(text).append("</a>");
		} else {
			b.append("/>");
		}
		return b.toString();
	}

	/**
	 * return a LI object
	 *
	 * @param value
	 * @return
	 */
	public static String intoLI(String value) {
		return intoTag("li", value);
	}

	/**
	 * return a paragraph (P) object
	 *
	 * @param text
	 * @param style
	 * @return
	 */
	public static String intoP(String text, String... style) {
		StringBuilder b = new StringBuilder("<p");
		if (style != null && style.length > 0) {
			b.append(" style=\"");
			for (String st : style) {
				b.append(st);
				if (!st.endsWith(";")) {
					b.append(";");
				}
			}
			b.append("\"");
		}
		b.append(">").append(text).append(P_END);
		return b.toString();
	}

	/**
	 * return a paragraph (P) object
	 *
	 * @param text
	 * @param style
	 * @return
	 */
	public static String intoPcenter(String text) {
		return intoP(text, ALIGN_CENTER);
	}

	/**
	 * return an italic (I) object
	 *
	 * @param text
	 * @return
	 */
	public static String intoI(String text) {
		return intoTag("i", text);
	}

	/**
	 * return an uderlined (U) object
	 *
	 * @param text
	 * @return
	 */
	public static String intoU(String text) {
		return intoTag("u", text);
	}

	/**
	 * return a bold (B) object
	 *
	 * @param text
	 * @return
	 */
	public static String intoB(String text) {
		return intoTag("b", text);
	}

	/**
	 * return an image tag
	 *
	 * @param src
	 * @param title
	 * @param alt
	 * @return
	 */
	public static String intoIMG(String src, String title, String alt, Integer... height) {
		StringBuilder b = new StringBuilder("<img");
		b.append(" src=\"").append(src).append("\"");
		if (!title.isEmpty()) {
			b.append(" title=\"").append(title).append("\"");
		}
		if (!alt.isEmpty()) {
			b.append(" alt=\"").append(alt).append("\"");
		} else {
			b.append(" alt=\"image\"");
		}
		if (height != null && height.length > 0) {
			b.append("width=\"").append(height[0].toString()).append("\"");
		}
		b.append(">");
		return b.toString();
	}

	/**
	 * return a fonted text with a given size
	 *
	 * @param size
	 * @param text
	 * @return
	 */
	public static String intoFont(int size, String text) {
		return "<font size=\"" + size + "\">" + text + "</font>";
	}

	/**
	 * return a fonted text with a given color
	 *
	 * @param color
	 * @param text
	 * @return
	 */
	public static String intoFontColor(String color, String text) {
		return "<font color=\"" + color + "\">" + text + "</font>";
	}

	/**
	 * return a small text
	 *
	 * @param text
	 * @return
	 */
	public static String intoSmall(String text) {
		return intoTag("small", text);
	}

	/**
	 * return a row (TR) for a TABLE
	 *
	 * @param text
	 * @param complement
	 * @return
	 */
	public static String intoTR(String text, String... complement) {
		return intoTag("tr", text, complement);
	}

	/**
	 * return a cell (TD) for a TABLE
	 *
	 * @param text
	 * @param complement
	 * @return
	 */
	public static String intoTD(String text, String... complement) {
		return intoTag("td", text, complement);
	}

	/**
	 * return a tagged text
	 *
	 * @param tag
	 * @param value
	 * @param opt: complement
	 * @return
	 */
	public static String intoTag(String tag, String value, String... opt) {
		StringBuilder r = new StringBuilder("<" + tag);
		if (opt != null && opt.length > 0) {
			for (String s : opt) {
				r.append(" ").append(s);
			}
		}
		r.append(">")
				.append(value)
				.append("</").append(tag).append(">");
		return r.toString();
	}

	/**
	 * return a text enclose into a TABLE
	 *
	 * @param text
	 * @return
	 */
	public static String wrapIntoTable(String text) {
		return wrapIntoTable(text, 200);
	}

	/**
	 * return a text enclose into a limited width TABLE
	 *
	 * @param text
	 * @param width
	 * @return
	 */
	public static String wrapIntoTable(String text, int width) {
		return BEGIN
				+ "<table width=\"" + width + "\">"
				+ intoTR(intoTD(text))
				+ TABLE_END;
	}

	/**
	 * compare two HTML text strings
	 *
	 * @param html1
	 * @param html2
	 * @return
	 */
	public static boolean equalsHtml(String html1, String html2) {
		String s1 = Jsoup.parse(html1).text();
		String s2 = Jsoup.parse(html2).text();
		return (s1.equals(s2));
	}

	/**
	 * return text len without tags and UTF8 codes
	 *
	 * @param html
	 * @return the len
	 */
	public static int getTextSize(String html) {
		if (html == null) {
			return (0);
		}
		String x = htmlToText(html).replaceAll("&#.*;", "?");
		return (x.length());
	}

	public static String htmlToText(String html) {
		return htmlToText(html, false);
	}

	/**
	 * convert a HTML String into truncated plain text
	 *
	 * @param html
	 * @param len
	 * @return
	 */
	public static String htmlToText(String html, int len) {
		String txt = html.replaceAll("<[/]?img[^>]*>", "[" + I18N.getMsg("image") + "]");
		return TextUtil.truncateString(htmlToText(txt, true), len);
	}

	/**
	 * convert HTML to plain text
	 *
	 * @param src String to convert
	 * @param preserveNewLines true for preserving new lines char
	 * @return
	 */
	public static String htmlToText(String src, boolean preserveNewLines) {
		if (src == null) {
			return ("");
		}
		String html = src;
		if (!preserveNewLines) {
			html = Jsoup.parse(html).text();
		} else {
			html = Jsoup.parse(html).wholeText();
		}
		html = html.replace("\n\n", "\n");
		while (html.startsWith("\n")) {
			html = html.substring(1);
		}
		html = TextUtil.trimText(html);
		while (html.endsWith("\n")) {
			html = html.substring(0, html.length() - 1);
		}
		return html;
	}

	/**
	 * convert any special chars to hexadecimal codes
	 *
	 * @param text
	 * @return
	 */
	public static String textToHtmlCode(String text) {
		String out = "";
		for (char ch : text.toCharArray()) {
			out += (ch < 128 ? ch : "&#" + (int) ch + ";");
		}
		return (out);
	}

	/**
	 * convert plain text to Html, inserting tags like P or BR
	 *
	 * @param text
	 * @return
	 */
	public static String textToHTML(String text) {
		if (text == null) {
			return null;
		}
		int length = text.length();
		boolean prevSlashR = false;
		StringBuilder out = new StringBuilder();
		for (int i = 0; i < length; i++) {
			char ch = text.charAt(i);
			switch (ch) {
				case '\r':
					if (prevSlashR) {
						out.append(BR);
					}
					prevSlashR = true;
					break;
				case '\n':
					prevSlashR = false;
					out.append(BR);
					break;
				case '"':
					if (prevSlashR) {
						out.append(BR);
						prevSlashR = false;
					}
					out.append("&quot;");
					break;
				case '<':
					if (prevSlashR) {
						out.append(BR);
						prevSlashR = false;
					}
					out.append("&lt;");
					break;
				case '>':
					if (prevSlashR) {
						out.append(BR);
						prevSlashR = false;
					}
					out.append("&gt;");
					break;
				case '&':
					if (prevSlashR) {
						out.append(BR);
						prevSlashR = false;
					}
					out.append("&amp;");
					break;
				default:
					if (prevSlashR) {
						out.append(BR);
						prevSlashR = false;
					}
					out.append(ch);
					break;
			}
		}
		return (out.toString());
	}

	/**
	 * return a HTML string into a row (TR) with 2 columns (TD)
	 *
	 * @param key
	 * @param val
	 * @return
	 */
	public static String getRow2Cols(String key, String val) {
		return TR + TD + I18N.getMsg(key) + TD_END + TD + val + TD_END + TR_END;
	}

	public static String getRow2Cols(String key, Integer val) {
		return TR + TD + I18N.getMsg(key) + TD_END + TD + String.format("%,d", val) + TD_END + TR_END;
	}

	/**
	 * return a SPAN color (a 4 colored spaces)
	 *
	 * @param clr
	 * @return
	 */
	public static String getColorSpan(Color clr, int nb) {
		String htmlClr = (clr == null ? "white" : "#" + ColorUtil.getHexName(clr));
		StringBuilder buf = new StringBuilder("<span style=\"");
		if (clr != null) {
			buf.append("background-color:").append(htmlClr).append(";");
		}
		buf.append("\">");
		for (int i = 0; i < nb; i++) {
			buf.append("&nbsp;");
		}
		buf.append("</span>");
		return buf.toString();
	}

	/**
	 * return a SPAN color (a 4 colored spaces)
	 *
	 * @param clr
	 * @return
	 */
	public static String getColorSpan(Color clr) {
		return getColorSpan(clr, 4);
	}

	/**
	 * return a colored bold title
	 *
	 * @param clr: the color
	 * @param title: the text title to colorize
	 * @return
	 */
	public static String getColoredTitle(Color clr, String title) {
		String htmlClr = (clr == null ? "white" : ColorUtil.getHexName(clr));
		String buf = "<div style=\"padding-top:2pt;padding-bottom:2pt;"
				+ "padding-left:4pt;padding-right:4pt;margin-bottom:2pt;";
		if (clr != null) {
			buf += "background-color:" + htmlClr + ";";
		}
		if (clr != null && ColorUtil.isDark(clr)) {
			buf += "color:white;";
		}
		buf += "\">" + intoB(title) + DIV_END;
		return (buf);
	}

	/**
	 * return the HEAD TITLE
	 *
	 * @param title
	 * @return
	 */
	public static String getHeadTitle(String title) {
		return intoTag("title", title) + "\n";
	}

	/**
	 * return a RED colored string
	 *
	 * @param str
	 * @return
	 */
	public static String getWarning(String str) {
		return intoFontColor("red", intoB(str));
	}

	/**
	 * return a horizontal rule (HR)
	 *
	 * @return
	 */
	public static String getHr() {
		return "<hr style=\"margin:10pt\" />";
	}

	private static final String REG_CRLF = "(\r\n|\r|\n|\n\r)";

	/**
	 * return a formatted warning text colored in red
	 *
	 * @param buf
	 * @param warning
	 * @return
	 */
	public static String appendFormatedWarning(String buf, String warning) {
		String str = warning.replaceAll(REG_CRLF, "<br>");
		if (str.isEmpty()) {
			return (buf);
		}
		return ("<div style=\"color:red\">" + str + DIV_END);
	}

	/**
	 * return a formatted description
	 *
	 * @param buf
	 * @param descr
	 * @param shorten
	 * @return
	 */
	public static String appendFormatedDescr(String buf, String descr, boolean shorten) {
		String str = descr.replaceAll(REG_CRLF, "<br>");
		if (str.isEmpty()) {
			return (buf);
		}
		String ret = buf;
		if (shorten) {
			ret += "<div style=\"width:300pt\">" + TextUtil.truncateString(str, 300);
		} else {
			ret += DIV + str;
		}
		return (ret + DIV_END);
	}

	/**
	 * return a formatted Notes
	 *
	 * @param buf
	 * @param notes
	 * @param shorten
	 * @return
	 */
	public static String appendFormatedNotes(String buf, String notes, boolean shorten) {
		String str = notes.replaceAll(REG_CRLF, "<br>");
		if (str.isEmpty()) {
			return (buf);
		}
		String ret = "<hr style=\"margin:5pt\"/>";
		if (shorten) {
			ret += "<div style=\"width:300pt\">" + TextUtil.truncateString(str, 300);
		} else {
			ret += DIV + str;
		}
		return (ret + DIV_END);
	}

	/**
	 * return a formatted informations
	 *
	 * @param buf
	 * @param metaInfos
	 * @return
	 */
	public static String appendFormatedMetaInfos(String buf, String metaInfos) {
		String str = metaInfos.replaceAll(REG_CRLF, "<br>");
		if (str.isEmpty()) {
			return (buf);
		}
		return ("<hr style=\"margin:5pt;\"/>" + DIV + str + DIV_END);
	}

	/**
	 * return a subtitle text string, level depends upon nb of points
	 *
	 * @param lib
	 * @return
	 */
	public static String getHtag(String lib) {
		String[] x = lib.split("\\.");
		int n = 2 + x.length;
		if (n > 6) {
			n = 6;
		}
		String t = lib;
		if (t.startsWith(".")) {
			t = lib.substring(1);
		}
		if (t.contains(". ")) {
			t = t.substring(t.indexOf(". ") + 2);
		}
		return ("<h" + n + ">" + t + "</h" + n + ">\n");
	}

	/**
	 * return an IMG object
	 *
	 * @param src
	 * @param alt
	 * @param width
	 * @param height
	 * @return
	 */
	public static String getImg(String src, String alt, String width, String height) {
		StringBuilder b = new StringBuilder("<img");
		b.append(" src=\"");
		b.append(src);
		b.append("\"");
		if (!alt.isEmpty()) {
			b.append(" alt=\"").append(alt).append("\"");
		}
		if (!width.isEmpty()) {
			b.append(" width=\"").append(width).append("\"");
		}
		if (!height.isEmpty()) {
			b.append(" height=\"").append(height).append("\"");
		}
		b.append(">");
		return b.toString();
	}

	/**
	 * return an indentation
	 *
	 * @param lev
	 * @return
	 */
	public static String getIndent(int lev) {
		String r = "";
		for (int i = 0; i < lev; i++) {
			r += "&nbsp;&nbsp;&nbsp;&nbsp;";
		}
		return (r);
	}

	/**
	 * return HTML Font definition
	 *
	 * @param font
	 * @return
	 */
	public static String fontToHtml(Font font) {
		String r = "style=\""
				+ FONT_FAMILY + ": " + font.getFontName() + ";"
				+ FONT_SIZE + ": " + font.getSize() + "pt;"
				+ "\"";
		return (r);
	}

	/**
	 * return a formatted key,value into one row (TR) of two columns (TD)
	 *
	 * @param key
	 * @param value
	 * @return
	 */
	public static String toHtml(String key, Integer value) {
		return toHtml(key, value.toString());
	}

	/**
	 * return a formatted key,value into one row (TR) of two columns (TD)
	 *
	 * @param key
	 * @param value
	 * @return
	 */
	public static String toHtml(String key, String value) {
		StringBuilder b = new StringBuilder();
		b.append(TR);
		b.append("<td valign=\"top\">")
				.append(intoB(I18N.getMsg(key)))
				.append(TD_END).append(TD);
		if (value == null) {
			b.append("null");
		} else if (value.isEmpty()) {
			b.append(I18N.getMsg("empty"));
		} else {
			b.append(value);
		}
		b.append(TD_END).append(TR_END);
		return (b.toString());
	}

	/**
	 * clean HTML for XHTML
	 *
	 * @param html: string to check/transform
	 * @return checked String
	 */
	public static String toXhtml(final String html) {
		final Document document = Jsoup.parse(html);
		document.outputSettings().syntax(org.jsoup.nodes.Document.OutputSettings.Syntax.xml);
		return document.html();
	}

	/**
	 * clean HTML for a XHTML body
	 *
	 * @param html: string to check/transform
	 * @return checked String
	 */
	public static String toXhtmlBody(final String html) {
		final Document document = Jsoup.parse(html);
		document.outputSettings().syntax(org.jsoup.nodes.Document.OutputSettings.Syntax.xml);
		return document.body().html();
	}

	public static final int EM_REMOVETAGS = 0, EM_REMOVEALL = 1, EM_LEFTASIS = 2;

	/**
	 * remove or set color EM tag
	 *
	 * @param text
	 * @param hi: 0=simply remove EM tags 1=remove EM (tag+text) 2=set EM tags to yellow one
	 * @return
	 */
	public static String emTag(String text, int hi) {
		switch (hi) {
			case 0:
				return (text.replaceAll("<em.*>", "").replace("</em>", ""));
			case 1:
				return (text.replaceAll("<em.*>.*</em>", ""));
			case 2:
				return text.replace("<em>", "<em style=\"background: yellow;font-style: normal;\">");
			default:
				break;
		}
		return (text);
	}

	/**
	 * return a replacing tag
	 *
	 * @param text : HTML code to search
	 * @param tag : tag to replace
	 * @param by : replacing tag
	 * @return : new HTML code
	 */
	public static String replaceTags(String text, String tag, String by) {
		if (text == null || text.isEmpty()) {
			return text;
		}
		String s = text.replaceAll("<" + tag.toUpperCase(), "<" + by.toUpperCase());
		s = s.replaceAll("</" + tag.toUpperCase(), "</" + by.toUpperCase());
		s = s.replaceAll("<" + tag.toLowerCase(), "<" + by.toLowerCase());
		return s.replace("</" + tag.toLowerCase(), "</" + by.toLowerCase());
	}

	/**
	 * tranform each fixed "file://" by a relative path
	 *
	 * @param html
	 * @param path
	 * @return
	 */
	public static String fileToRel(String html, String path) {
		if (!html.contains("file:")) {
			return html;
		}
		String r = "file://" + path;
		String s = html.replace(r, "file:..");
		return s;
	}

	/**
	 * tranform each relative path by a fixed "file://"
	 *
	 * @param html
	 * @param path
	 * @return
	 */
	public static String fileToPath(String html, String path) {
		if (!html.contains("file:")) {
			return html;
		}
		String r = "file:..";
		return html.replace(r, "file://" + path);
	}

}
